/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { IIdCompressor, StableId } from "@fluidframework/id-compressor/internal";
import { LocalNodeKey, StableNodeKey } from "./nodeKey.js";
/**
 * An object which handles the generation of node keys as well as conversion between their two types ({@link StableNodeKey} and {@link LocalNodeKey}).
 * @internal
 */
export interface NodeKeyManager {
    /**
     * Generate a {@link StableNodeKey}.
     */
    generateLocalNodeKey(): LocalNodeKey;
    /**
     * Convert the given {@link StableNodeKey} into its {@link LocalNodeKey} form.
     */
    localizeNodeKey(key: StableNodeKey): LocalNodeKey;
    /**
     * Convert the given {@link LocalNodeKey} into its {@link StableNodeKey} form.
     */
    stabilizeNodeKey(key: LocalNodeKey): StableNodeKey;
    /**
     * Attempts to recompress a StableNodeKey.
     * @param key - The key that is attempted to recompress.
     * @returns The `LocalNodeKey` associated with `key` or undefined if the key was not generated by any session known to this compressor.
     */
    tryLocalizeNodeKey(key: string): LocalNodeKey | undefined;
}
/**
 * Creates a {@link NodeKeyManager} from the given {@link IIdCompressor}.
 * @param idCompressor - the compressor to use for key generation, compression, and decompression.
 * If undefined, then attempts to generate or convert keys will throw an error.
 */
export declare function createNodeKeyManager(idCompressor?: IIdCompressor | undefined): NodeKeyManager;
export declare function isStableNodeKey(key: string): key is StableNodeKey;
/**
 * Create a {@link NodeKeyManager} that generates deterministic {@link StableNodeKey}s and {@link LocalNodeKey}s.
 * @remarks This is useful for test environments because it will always yield the same keys in the same order.
 * It should not be used for production environments for the same reason; the {@link StableNodeKey}s are not universally unique.
 */
export declare function createMockNodeKeyManager(): MockNodeKeyManager;
declare class MockNodeKeyManager implements NodeKeyManager {
    private count;
    generateStableNodeKey(): StableNodeKey;
    generateLocalNodeKey(): LocalNodeKey;
    localizeNodeKey(key: StableNodeKey): LocalNodeKey;
    stabilizeNodeKey(key: LocalNodeKey): StableNodeKey;
    tryLocalizeNodeKey(key: string): LocalNodeKey | undefined;
    getId(offset: number): StableId;
}
export {};
//# sourceMappingURL=nodeKeyManager.d.ts.map